@@ -1,3 +1,22 @@
+/*!
+A system for parsing entire nodes and node lists. Unlike the rest of
+`kaydle-primitives`, this module doesn't expose ordinary nom parsers and
+container types. Instead, because it's designed specifically to support the
+implementation of a serde deserializer, it exposes a set of "processors", which
+can be used to fetch nodes or the contents of nodes step-by-step.
+
+This module makes extensive use of the builder traits defined in
+`kaydle-primitives` (such as [`ValueBuilder`] and [`StringBuilder`]) to allow
+callers to precisely control how much information they need from the node.
+Often you can use `()` instead of a real KDL type if you don't care about a
+value; this will be faster to parse.
+
+This module tries to be as misuse resistant as possible, using borrowing and
+move semantics to ensure that methods aren't called out of order. Where
+build-time correctness is impossible, it instead uses runtime tracking and
+panics to ensure consistent state.
+*/
+
 use std::char::CharTryFromError;
 
 use nom::{
@@ -10,9 +29,9 @@ use nom::{
 use nom_supreme::{tag::TagError, ParserExt};
 
 use crate::{
-    annotation::{with_annotation, AnnotationBuilder, GenericAnnotated, RecognizedAnnotation},
+    annotation::{with_annotation, AnnotationBuilder, GenericAnnotated},
     number::BoundsError,
-    property::{parse_property, GenericProperty, RecognizedProperty},
+    property::{parse_property, GenericProperty},
     string::{parse_identifier, StringBuilder},
     value::{parse_value, ValueBuilder},
     whitespace::{parse_linespace, parse_node_space, parse_node_terminator},
@@ -55,11 +74,14 @@ where
 /// annotated name of the node as well as a node processor for fetching
 /// arguments, properties, and children from the node.
 pub struct NodeItem<'i, 'a, A, T> {
+    /// The (possibly annotated) name of this node
     pub name: GenericAnnotated<A, T>,
-    pub content: NodeProcessor<'i, 'a>,
+
+    /// A processor, used for getting the contents of the node.
+    pub node: NodeProcessor<'i, 'a>,
 }
 
-type EmptyNodeItem<'i, 'a> = NodeItem<'i, 'a, (), ()>;
+type RecognizedNodeItem<'i, 'a> = NodeItem<'i, 'a, (), ()>;
 
 /// Trait for types that can parse a node list. Abstracts over a node document
 /// processor, which operates at the top level, and a node children processor,
@@ -90,7 +112,7 @@ pub trait NodeListProcessor<'i, 'p>: Sized {
         E: FromExternalError<&'i str, BoundsError>,
         E: ContextError<&'i str>,
     {
-        while let Some(EmptyNodeItem { content: node, .. }) = self.next_node()? {
+        while let Some(RecognizedNodeItem { node, .. }) = self.next_node()? {
             node.drain()?;
         }
 
@@ -98,15 +120,25 @@ pub trait NodeListProcessor<'i, 'p>: Sized {
     }
 }
 
-/// Processor for a top level kdl document.
+/// Processor for a top level kdl document. Returns the nodes in the document.
 #[derive(Debug, Clone)]
 pub struct NodeDocumentProcessor<'i> {
+    /// The original input string
     state: &'i str,
+
+    /// Bool that ensures that node processors fully consume their nodes, so
+    /// that parse state remains consistent. Set to true when a node processor
+    /// is returned, and only resets to false when that processor is finished.
+    child_in_progress: bool,
 }
 
 impl<'i> NodeDocumentProcessor<'i> {
+    /// Create a new `NodeDocumentProcessor` from an input string.
     pub fn new(input: &'i str) -> Self {
-        Self { state: input }
+        Self {
+            state: input,
+            child_in_progress: false,
+        }
     }
 
     fn run_parser<T, E>(&mut self, parser: impl Parser<&'i str, T, E>) -> Result<T, NomErr<E>> {
@@ -124,13 +156,24 @@ impl<'i, 'p> NodeListProcessor<'i, 'p> for NodeDocumentProcessor<'i> {
         E: FromExternalError<&'i str, CharTryFromError>,
         E: ContextError<&'i str>,
     {
+        if self.child_in_progress {
+            panic!(
+                "Called next_node before the previous node was fully parsed. This is a kaydle bug."
+            )
+        }
+
         self.run_parser(parse_node_start(eof.value(())))
             .map(move |node_name| {
-                node_name.map(move |name| NodeItem {
-                    name,
-                    content: NodeProcessor {
-                        state: &mut self.state,
-                    },
+                node_name.map(move |name| {
+                    self.child_in_progress = true;
+
+                    NodeItem {
+                        name,
+                        node: NodeProcessor {
+                            state: &mut self.state,
+                            in_progress: &mut self.child_in_progress,
+                        },
+                    }
                 })
             })
     }
@@ -143,32 +186,39 @@ enum InternalNodeEvent<VA, V, K, PA, P> {
     End,
 }
 
-/// A piece of content from a node
+/// A piece of content from a node.
 #[derive(Debug)]
 pub enum NodeEvent<'i, 'p, VA, V, K, PA, P> {
     /// An argument from a node
-    Argument(
-        /// The argument
-        GenericAnnotated<VA, V>,
+    Argument {
+        /// The value, with its annotation
+        argument: GenericAnnotated<VA, V>,
         /// The processor containing the rest of the node
-        NodeProcessor<'i, 'p>,
-    ),
+        tail: NodeProcessor<'i, 'p>,
+    },
 
     /// A property (key-value pair) from a node
-    Property(
+    Property {
         /// The property
-        GenericProperty<K, PA, P>,
+        property: GenericProperty<K, PA, P>,
         /// The processor containing the rest of the node
-        NodeProcessor<'i, 'p>,
-    ),
+        tail: NodeProcessor<'i, 'p>,
+    },
 
     /// A set of children from the node.
-    Children(NodeChildrenProcessor<'i, 'p>),
+    Children {
+        /// A `NodeListProcessor` used to get child nodes one-by-one
+        children: NodeChildrenProcessor<'i, 'p>,
+    },
 
     /// There was nothing else in the node.
     End,
 }
 
+/// A [`NodeEvent`] containing no data. Used when the caller care what _kind_ of
+/// thing was in the node, but not the actual value / content.
+pub type RecognizedNodeEvent<'i, 'p> = NodeEvent<'i, 'p, (), (), (), (), ()>;
+
 fn parse_node_event<'i, E, VA, V, K, PA, P>(
     input: &'i str,
 ) -> IResult<&'i str, InternalNodeEvent<VA, V, K, PA, P>, E>
@@ -184,13 +234,21 @@ where
     E: FromExternalError<&'i str, BoundsError>,
     E: ContextError<&'i str>,
 {
+    // There are two major improvements that can be made to this parser, both
+    // related to duplicate work:
+    //
+    // - Don't repeat the `.preceded_by(parse_node_space)` between the
+    //   argument/property case and the children/end case
+    // - Don't repeat the string parse for a value and the string parse for
+    //   a property.
+    //
+    // For now we assume, perhaps aspirationally, that the optimizer notices
+    // and unifies some of the commonalities.
     alt((
         // Parse a value or property, preceded by 1 or more whitespace
         alt((
             // Important: make sure to try to parse a property first, since
             // "abc"=10 could be conservatively parsed as just the value "abc"
-            // TODO: try to parse a value first, and if it's a string, try to
-            // parse =value (in other words, avoid duplicating the string parse)
             parse_property
                 .map(InternalNodeEvent::Property)
                 .context("property"),
@@ -218,30 +276,22 @@ where
 #[derive(Debug)]
 pub struct NodeProcessor<'i, 'p> {
     state: &'p mut &'i str,
+
+    /// Bool owned by the parent's list processor. Must be set to false only when
+    /// this node has been fully consumed.
+    in_progress: &'p mut bool,
 }
 
 impl<'i, 'p> NodeProcessor<'i, 'p> {
-    // Parse and discard everything in this node
-    pub fn drain<E>(mut self) -> Result<(), NomErr<E>>
-    where
-        E: ParseError<&'i str>,
-        E: TagError<&'i str, &'static str>,
-        E: FromExternalError<&'i str, CharTryFromError>,
-        E: FromExternalError<&'i str, BoundsError>,
-        E: ContextError<&'i str>,
-    {
-        loop {
-            self = match self.next_event()? {
-                NodeEvent::Argument(RecognizedAnnotation { .. }, next) => next,
-                NodeEvent::Property(RecognizedProperty { .. }, next) => next,
-                NodeEvent::Children(children) => break children.drain(),
-                NodeEvent::End => break Ok(()),
-            }
-        }
-    }
-}
-
-impl<'i, 'p> NodeProcessor<'i, 'p> {
+    /// Get the next piece of content from a node. This can be an argument,
+    /// a property, a set of children, or [`End`][NodeEvent::End] if the node
+    /// is done.
+    ///
+    /// For correctness, this method is move oriented. If the event is an
+    /// argument or property, the event includes a new [`NodeProcessor`] for
+    /// fetching the rest of the node. Conversely, if the event is children or
+    /// the end of the node, the processor is consumed, because there's nothing
+    /// more that can be retrieved from this node.
     pub fn next_event<VA, V, K, PA, P, E>(
         self,
     ) -> Result<NodeEvent<'i, 'p, VA, V, K, PA, P>, NomErr<E>>
@@ -257,20 +307,67 @@ impl<'i, 'p> NodeProcessor<'i, 'p> {
         E: FromExternalError<&'i str, BoundsError>,
         E: ContextError<&'i str>,
     {
+        // Because we use a move-oriented interface, there's no need to check
+        // in_progress. We (or the children processor we return) just need to
+        // make sure it's reset to false when we're done.
         run_parser_on(self.state, parse_node_event).map(move |event| match event {
-            InternalNodeEvent::Argument(value) => NodeEvent::Argument(value, self),
-            InternalNodeEvent::Property(prop) => NodeEvent::Property(prop, self),
-            InternalNodeEvent::Children => NodeEvent::Children(NodeChildrenProcessor {
-                state: Some(self.state),
-            }),
-            InternalNodeEvent::End => NodeEvent::End,
+            InternalNodeEvent::Argument(argument) => NodeEvent::Argument {
+                argument,
+                tail: self,
+            },
+            InternalNodeEvent::Property(property) => NodeEvent::Property {
+                property,
+                tail: self,
+            },
+            InternalNodeEvent::Children => NodeEvent::Children {
+                children: NodeChildrenProcessor {
+                    state: self.state,
+                    in_progress: self.in_progress,
+                    child_in_progress: false,
+                },
+            },
+            InternalNodeEvent::End => {
+                *self.in_progress = false;
+                NodeEvent::End
+            }
         })
     }
+
+    /// Parse and discard everything in this node
+    pub fn drain<E>(mut self) -> Result<(), NomErr<E>>
+    where
+        E: ParseError<&'i str>,
+        E: TagError<&'i str, &'static str>,
+        E: FromExternalError<&'i str, CharTryFromError>,
+        E: FromExternalError<&'i str, BoundsError>,
+        E: ContextError<&'i str>,
+    {
+        loop {
+            self = match self.next_event()? {
+                RecognizedNodeEvent::Argument { tail, .. } => tail,
+                RecognizedNodeEvent::Property { tail, .. } => tail,
+                RecognizedNodeEvent::Children { children } => break children.drain(),
+                RecognizedNodeEvent::End => break Ok(()),
+            }
+        }
+    }
 }
 
+/// Processor for child nodes of a particular node (contained in `{ }`).
+/// Returns the child nodes.
 #[derive(Debug)]
 pub struct NodeChildrenProcessor<'i, 'p> {
-    state: Option<&'p mut &'i str>,
+    state: &'p mut &'i str,
+
+    /// Bool owned by the parent's list processor. Must be set to true only when
+    /// this list has been fully consumed. Additionally used to ensure that
+    /// `next_node` exhibits fused behavior.
+    in_progress: &'p mut bool,
+
+    /// Bool that ensures that node processors fully consume their nodes, so
+    /// that parse state remains consistent. Set to false when a node processor
+    /// is returns, and only resets to true when that processor is finished.
+    child_in_progress: bool,
 }
 
 impl<'i, 'p> NodeListProcessor<'i, 'p> for NodeChildrenProcessor<'i, 'p> {
@@ -284,17 +381,38 @@ impl<'i, 'p> NodeListProcessor<'i, 'p> for NodeChildrenProcessor<'i, 'p> {
         E: FromExternalError<&'i str, CharTryFromError>,
         E: ContextError<&'i str>,
     {
-        match self.state.take() {
-            None => Ok(None),
-            Some(state) => match run_parser_on(state, parse_node_start(char('}').value(())))? {
-                None => Ok(None),
-                Some(name) => Ok(Some(NodeItem {
+        // If the *parent* is at a node boundary, it means that *this*
+        // set of children was previously completed.
+        if !*self.in_progress {
+            return Ok(None);
+        }
+
+        // We *must* be at a node boundary internally; if we're not, it means one of our
+        // children wasn't fully consumed.
+        if self.child_in_progress {
+            panic!(
+                "Called next_node before the previous node was fully parsed. This is a kaydle bug."
+            )
+        }
+
+        match run_parser_on(self.state, parse_node_start(char('}').value(())))? {
+            // None here means that we successfully parsed the end-of-children. Inform the parent.
+            None => {
+                *self.in_progress = false;
+                Ok(None)
+            }
+            Some(name) => {
+                self.child_in_progress = true;
+                eprintln!("Entering child");
+
+                Ok(Some(NodeItem {
                     name,
-                    content: NodeProcessor {
-                        state: *self.state.insert(state),
+                    node: NodeProcessor {
+                        state: self.state,
+                        in_progress: &mut self.child_in_progress,
                     },
-                })),
-            },
+                }))
+            }
         }
     }
 }
