@@ -1,3 +1,7 @@
+/*!
+Parsers and utility types for parsing KDL strings and identifiers.
+ */
+
 use std::{
     borrow::Cow,
     char::CharTryFromError,
@@ -24,37 +28,42 @@ use nom_supreme::{
 };
 use serde::{de, Deserialize, Serialize};
 
-use crate::util::back;
-
 /// A KDL string, parsed from an identifier, escaped string, or raw string.
 /// Exists in either Owned or Borrowed form, depending on whether there were
-/// excapes in the string
+/// escapes in the string. Doesn't track the origin of the string (identifier,
+/// escaped, or raw), because KDL semantics treat them all identically.
 #[derive(Debug, Clone, Eq)]
 pub struct KdlString<'a> {
     inner: Cow<'a, str>,
 }
 
 impl<'a> KdlString<'a> {
+    /// Create a new, empty KDL String
     pub fn new() -> Self {
         Self::from_borrowed("")
     }
 
+    /// Create a KDL string from a `Cow<str>`.
     pub fn from_cow(cow: Cow<'a, str>) -> Self {
         Self { inner: cow }
     }
 
+    /// Create a borrowed KDL string
     pub fn from_borrowed(s: &'a str) -> Self {
         Self::from_cow(Cow::Borrowed(s))
     }
 
+    /// Create an owned KDL string
     pub fn from_string(s: String) -> Self {
         Self::from_cow(Cow::Owned(s))
     }
 
+    /// Convert this unconditionally into an owned string.
     pub fn into_string(self) -> String {
         self.inner.into_owned()
     }
 
+    /// Get the `&str` contained in this string.
     pub fn as_str(&self) -> &str {
         self
     }
@@ -280,10 +289,6 @@ pub fn parse_raw_string<'i, E: ParseError<&'i str>>(input: &'i str) -> IResult<&
 where
     E: ParseError<&'i str>,
 {
-    // The very end the input (an empty string), used for certain error
-    // reports
-    let eof_location = back(input);
-
     let (input, hash_count) =
         parse_separated_terminated(char('#'), success(()), char('"'), || 0, |n, _c| n + 1)
             .or(char('"').value(0))
@@ -292,15 +297,10 @@ where
 
     let mut shifter = SliceShifter::new(input);
 
-    'outer: loop {
+    loop {
         match memchr(b'"', shifter.tail().as_bytes()) {
             // Couldn't find any quotes; need more input
-            None => {
-                return Err(NomErr::Error(E::or(
-                    make_error(eof_location, ErrorKind::Eof),
-                    E::from_char(eof_location, '"'),
-                )))
-            }
+            None => return Err(NomErr::Error(make_error("", ErrorKind::Eof))),
 
             // Found a quote; search the successor bytes for hashes
             Some(quote_idx) => {
@@ -308,20 +308,20 @@ where
                 let payload = shifter.head();
                 shifter.shift(1);
 
-                for _ in 0..hash_count {
-                    match shifter.tail().as_bytes().get(0) {
-                        None => {
-                            return Err(NomErr::Error(E::or(
-                                make_error(eof_location, ErrorKind::Eof),
-                                E::from_char(eof_location, '#'),
-                            )))
+                match shifter.tail().as_bytes().get(0..hash_count) {
+                    // Bounds error here means the input isn't large enough to
+                    // contain the hash bytes; this is an unexpected EoF
+                    None => return Err(NomErr::Error(make_error("", ErrorKind::Eof))),
+
+                    // Found our chunk; if it's all hashes, this is the end of
+                    // the string
+                    Some(chunk) => {
+                        if chunk.iter().all(|&b| b == b'#') {
+                            shifter.shift(hash_count);
+                            return Ok((shifter.tail(), payload));
                         }
-                        Some(b'#') => shifter.shift(1),
-                        Some(..) => continue 'outer,
                     }
                 }
-
-                return Ok((shifter.tail(), payload));
             }
         }
     }
@@ -472,14 +472,10 @@ fn parse_unescaped_chunk<'i, E>(input: &'i str) -> IResult<&'i str, &'i str, E>
 where
     E: ParseError<&'i str>,
 {
-    // The very end the input (an empty string), used for certain error
-    // reports
-    let eof_location = back(input);
-
     match memchr2(b'"', b'\\', input.as_bytes()) {
         None => Err(NomErr::Error(E::or(
-            make_error(eof_location, ErrorKind::Eof),
-            E::from_char(eof_location, '"'),
+            make_error("", ErrorKind::Eof),
+            E::from_char("", '"'),
         ))),
 
         Some(0) => Err(NomErr::Error(make_error(input, ErrorKind::TakeWhile1))),
